@@ -98,10 +98,10 @@ def find_awg(
                 params["awg_size"] = int(force_awg)
             elif limit_awg is not None:
                 params["max_awg"] = int(limit_awg)
-            gw.debug(f"AWG base data SQL: {sql.strip()}, params: {params}")
+            gw.verbose(f"AWG base data SQL: {sql.strip()}, params: {params}")
             cursor.execute(sql, params)
             rows = cursor.fetchall()
-            gw.debug(f"AWG base rows fetched: {rows}")
+            gw.verbose(f"AWG base rows fetched: {rows}")
 
             awg_data = {}
             for awg_size, line_num, k_ohm, a60, a75, a90 in rows:
@@ -144,7 +144,7 @@ def find_awg(
 
                     vdrop = base_vdrop * base["k"] / n
                     perc = vdrop / volts
-                    gw.debug(
+                    gw.verbose(
                         f"Eval AWG={awg_size} lines={n} drop={vdrop:.4f} perc={perc*100:.4f}%"
                     )
                     result = {
@@ -167,7 +167,7 @@ def find_awg(
                                 fill = find_conduit(AWG(awg_size), n * (phases + ground), conduit=c)
                                 result["conduit"] = c
                                 result["pipe_inch"] = fill["size_inch"]
-                            gw.debug(f"Selected cable result: {result}")
+                            gw.verbose(f"Selected cable result: {result}")
                             return result
                         if perc < best_perc:
                             best = result
@@ -179,7 +179,7 @@ def find_awg(
                                 fill = find_conduit(AWG(awg_size), n * (phases + ground), conduit=c)
                                 result["conduit"] = c
                                 result["pipe_inch"] = fill["size_inch"]
-                            gw.debug(f"Selected cable result: {result}")
+                            gw.verbose(f"Selected cable result: {result}")
                             return result
                         if perc < best_perc:
                             best = result
@@ -252,15 +252,15 @@ def find_conduit(awg, cables, *, conduit="emt"):
         return {"size_inch": size}
 
 
-def view_cable_finder(
+def view_awg_calculator(
     *, meters=None, amps="40", volts="220", material="cu",
     max_lines="1", max_awg=None, phases="1", temperature=None,
     conduit=None, ground="1", neutral="0", **kwargs
 ):
-    """Page builder for AWG cable finder with HTML form and result."""
+    """Page builder for AWG calculator with HTML form and result."""
     if not meters:
         return '''<link rel="stylesheet" href="/static/awg/cable_finder.css">
-            <h1>AWG Cable & Conduit Finder</h1>
+            <h1>Cable & Conduit Calculator</h1>
             <form method="post" class="cable-form">
                 <table class="form-table two-col">
                     <tr>
@@ -339,8 +339,13 @@ def view_cable_finder(
                             </select>
                         </td>
                     </tr>
+                    <tr>
+                        <td>
+                            <button type="submit" class="submit">Calculate</button>
+                        </td>
+                        <td></td>
+                    </tr>
                 </table>
-                <button type="submit" class="submit">Find Cable</button>
             </form>
         '''
     if max_awg in (None, ""):
@@ -358,19 +363,19 @@ def view_cable_finder(
             conduit=conduit_arg, ground=ground,
         )
     except Exception as e:
-        return f"<p class='error'>Error: {e}</p><p><a href='/awg/cable-finder'>&#8592; Try again</a></p>"
+        return f"<p class='error'>Error: {e}</p><p><a href='/awg/awg-calculator'>&#8592; Try again</a></p>"
 
     if result.get("awg") == "n/a":
         return """
             <h1>No Suitable Cable Found</h1>
             <p>No cable was found that meets the requirements within a 3% voltage drop.<br>
             Try adjusting the <b>cable size, amps, length, or material</b> and try again.</p>
-            <p><a href="/awg/cable-finder">&#8592; Calculate again</a></p>
+            <p><a href="/awg/awg-calculator">&#8592; Calculate again</a></p>
         """
 
     return f"""
         <div class='calc-result'>
-        <h1>AWG Cable & Conduit Results <img src='/static/awg/sponsor_logo.svg' alt='Sponsor Logo' class='sponsor-logo'></h1>
+        <h1>Calculator Results <img src='/static/awg/sponsor_logo.svg' alt='Sponsor Logo' class='sponsor-logo'></h1>
         <ul class='result-list'>
             <li><strong>AWG Size:</strong> {result['awg']}</li>
             <li><strong>Lines:</strong> {result['lines']}</li>
@@ -387,6 +392,6 @@ def view_cable_finder(
         <a href="https://www.gelectriic.com" target="_blank">Gelectriic Solutions</a></strong> for their
         support in creating this calculator.</em>
         </p>
-        <p><a href="/awg/cable-finder">&#8592; Calculate again</a></p>
+        <p><a href="/awg/awg-calculator">&#8592; Calculate again</a></p>
         </div>
     """
