@@ -3,13 +3,14 @@
 """
 GWAY NMCLI Network Monitor Project
 
-Single-run monitor and render functions for Linux systems using nmcli.
-Works with monitor/monitor.py. All state is read/written via gw.monitor.get_state/set_states('nmcli', {...}).
+Network monitoring helpers for Linux systems using ``nmcli``.
+All state is read/written via ``gw.monitor.get_state/set_states('nmcli', {...})``.
+The monitors only collect information and never modify the network.
 
 Monitors:
-    - monitor_nmcli: Full network check/fallback logic (AP/station/repair).
-    - monitor_ap_only: Ensure wlan0 is in AP mode.
-    - monitor_station_only: Ensure wlan0 is in station/client mode.
+    - monitor_nmcli: Collect information about all interfaces.
+    - monitor_ap_only: Record wlan0 status without changes.
+    - monitor_station_only: Same as above for station mode.
 
 Renders:
     - render_nmcli: Main network diagnostic report (HTML).
@@ -18,7 +19,11 @@ Renders:
 """
 
 import subprocess
+import shlex
+import html
+from bottle import request
 from gway import gw
+from gway.sigils import _unquote
 
 def now_iso():
     import datetime
@@ -30,6 +35,23 @@ def nmcli(*args):
     result = subprocess.run(["nmcli", *args], capture_output=True, text=True)
     return result.stdout.strip()
 
+def nmcli_list_connections():
+    """Return a list of (name, uuid, type, device) tuples from nmcli."""
+    output = nmcli('-t', '-f', 'NAME,UUID,TYPE,DEVICE', 'connection', 'show')
+    conns = []
+    for line in output.splitlines():
+        if not line:
+            continue
+        parts = line.split(':')
+        if len(parts) < 4:
+            parts += [''] * (4 - len(parts))
+        name, uuid, ctype, device = parts[:4]
+        conns.append((name, uuid, ctype, device))
+    return conns
+
+def _sanitize(val):
+    return _unquote(val.strip()) if isinstance(val, str) else val
+
 def get_wlan_ifaces():
     output = nmcli("device", "status")
     wlans = []
@@ -142,32 +164,21 @@ def get_wlan_status(iface):
             return status
     return {"ssid": None, "connected": False, "inet": False}
 
-def check_eth0_gateway():
+def gather_eth0_status():
+    """Record eth0 IP and whether a default gateway exists."""
     try:
         routes = subprocess.check_output(["ip", "route", "show", "dev", "eth0"], text=True)
         ip_addr = get_eth0_ip()
         state_update = {
             "eth0_ip": ip_addr,
-            "eth0_gateway": "default" in routes
+            "eth0_gateway": "default" in routes,
         }
-        if "default" in routes:
-            subprocess.run(["ip", "route", "del", "default", "dev", "eth0"], stderr=subprocess.DEVNULL)
-            nmcli("connection", "modify", "eth0", "ipv4.never-default", "yes")
-            nmcli("connection", "up", "eth0")
-            state_update.update({
-                "last_config_change": now_iso(),
-                "last_config_action": "Removed eth0 default route"
-            })
         gw.monitor.set_states('nmcli', state_update)
     except Exception as e:
-        gw.monitor.set_states('nmcli', {"last_error": f"eth0 gateway: {e}"})
+        gw.monitor.set_states('nmcli', {"last_error": f"eth0 status: {e}"})
 
 def ap_profile_exists(ap_con, ap_ssid, ap_password):
-    conns = nmcli("connection", "show")
-    for line in conns.splitlines():
-        fields = line.split()
-        if len(fields) < 4: continue
-        name, uuid, ctype, device = fields[:4]
+    for name, uuid, ctype, device in nmcli_list_connections():
         if name == ap_con and ctype == "wifi":
             details = nmcli("connection", "show", name)
             details_dict = {}
@@ -180,7 +191,10 @@ def ap_profile_exists(ap_con, ap_ssid, ap_password):
             return ssid_ok and pwd_ok
     return False
 
-def ensure_ap_profile(ap_con, ap_ssid, ap_password):
+def ensure_ap_profile(ap_con, ap_ssid, ap_password, ap_ip=None):
+    ap_con = _sanitize(ap_con)
+    ap_ssid = _sanitize(ap_ssid)
+    ap_password = _sanitize(ap_password)
     if not ap_con:
         raise ValueError("AP_CON must be specified.")
     if not ap_ssid or not ap_password:
@@ -198,12 +212,23 @@ def ensure_ap_profile(ap_con, ap_ssid, ap_password):
     gw.info(f"[nmcli] Creating AP profile: name={ap_con} ssid={ap_ssid}")
     nmcli("connection", "add", "type", "wifi", "ifname", "wlan0",
           "con-name", ap_con, "autoconnect", "no", "ssid", ap_ssid)
-    nmcli("connection", "modify", ap_con,
-          "mode", "ap", "802-11-wireless.band", "bg",
-          "wifi-sec.key-mgmt", "wpa-psk",
-          "wifi-sec.psk", ap_password)
+
+    local_ip = _sanitize(gw.resolve('[AP_GATEWAY]', default=ap_ip or '10.42.0.1'))
+    mod_args = [
+        "mode", "ap", "802-11-wireless.band", "bg",
+        "wifi-sec.key-mgmt", "wpa-psk",
+        "wifi-sec.psk", ap_password,
+        "ipv4.method", "shared",
+    ]
+    if local_ip:
+        mod_args += ["ipv4.addresses", f"{local_ip}/24"]
+    nmcli("connection", "modify", ap_con, *mod_args)
+    
 
 def set_wlan0_ap(ap_con, ap_ssid, ap_password):
+    ap_con = _sanitize(ap_con)
+    ap_ssid = _sanitize(ap_ssid)
+    ap_password = _sanitize(ap_password)
     ensure_ap_profile(ap_con, ap_ssid, ap_password)
     gw.info(f"[nmcli] Activating wlan0 AP: conn={ap_con}, ssid={ap_ssid}")
     nmcli("device", "disconnect", "wlan0")
@@ -230,8 +255,10 @@ def maybe_notify_ap_switch(ap_ssid, email=None):
     prev_mode = state.get("wlan0_mode")
     prev_ssid = state.get("wlan0_ssid")
     prev_inet = state.get("wlan0_inet")
-    recipient = email or gw.resolve('[ADMIN_EMAIL]')
-    if recipient and prev_mode == "station" and prev_inet:
+    recipient = email if email else gw.resolve('[ADMIN_EMAIL]', default=None)
+    if not recipient:
+        return
+    if prev_mode == "station" and prev_inet:
         subject = "[nmcli] wlan0 switching to AP mode"
         body = (
             f"Previous mode: station\n"
@@ -247,12 +274,7 @@ def maybe_notify_ap_switch(ap_ssid, email=None):
 
 def clean_and_reconnect_wifi(iface, ssid, password=None):
     gw.debug(f"[nmcli] clean_and_reconnect_wifi({iface}, ssid={ssid})")
-    conns = nmcli("connection", "show")
-    for line in conns.splitlines():
-        fields = line.split()
-        if len(fields) < 4:
-            continue
-        name, uuid, conn_type, device = fields[:4]
+    for name, uuid, conn_type, device in nmcli_list_connections():
         if conn_type == "wifi" and (device == iface or name == ssid):
             gw.info(f"[nmcli] Removing stale connection {name} ({uuid}) on {iface}")
             nmcli("connection", "down", name)
@@ -282,8 +304,7 @@ def try_connect_wlan0_known_networks():
 
     Returns the SSID if connection succeeds, otherwise None.
     """
-    conns = nmcli("connection", "show")
-    wifi_conns = [line.split()[0] for line in conns.splitlines()[1:] if "wifi" in line]
+    wifi_conns = [name for name, _, ctype, _ in nmcli_list_connections() if ctype == "wifi"]
     gw.debug(f"[nmcli] known wifi profiles: {wifi_conns}")
     for conn in wifi_conns:
         gw.info(f"[nmcli] Trying wlan0 connect: {conn}")
@@ -316,14 +337,7 @@ def try_connect_wlan0_known_networks():
 # --- Main single-run monitor functions ---
 
 def monitor_nmcli(**kwargs):
-    ap_ssid = kwargs.get("ap_ssid") or gw.resolve('[AP_SSID]')
-    ap_con = kwargs.get("ap_con") or ap_ssid or gw.resolve('[AP_CON]')
-    ap_password = kwargs.get("ap_password") or gw.resolve('[AP_PASSWORD]')
-    email = kwargs.get("email")
-    if not ap_con:
-        raise ValueError("Missing ap_con (AP_CON). Required for AP operation.")
-
-    check_eth0_gateway()
+    gather_eth0_status()
     wlan_ifaces = get_wlan_ifaces()
     gw.info(f"[nmcli] WLAN ifaces detected: {wlan_ifaces}")
     wlanN = {}
@@ -334,53 +348,24 @@ def monitor_nmcli(**kwargs):
         s = get_wlan_status(iface)
         wlanN[iface] = s
         gw.info(f"[nmcli] {iface} status: {s}")
-        if s["inet"]:
-            gw.info(f"[nmcli] {iface} has internet, keeping wlan0 as AP ({ap_ssid})")
-            maybe_notify_ap_switch(ap_ssid, email)
-            set_wlan0_ap(ap_con, ap_ssid, ap_password)
+        if s["inet"] and not found_inet:
             found_inet = True
             internet_iface = iface
             internet_ssid = s.get("ssid")
-            break
-        else:
-            clean_and_reconnect_wifi(iface, s.get("ssid") or iface)
-            s2 = get_wlan_status(iface)
-            wlanN[iface] = s2
-            if s2["inet"]:
-                gw.info(f"[nmcli] {iface} internet works after reset")
-                maybe_notify_ap_switch(ap_ssid, email)
-                set_wlan0_ap(ap_con, ap_ssid, ap_password)
-                found_inet = True
-                internet_iface = iface
-                internet_ssid = s2.get("ssid")
-                break
     gw.monitor.set_states('nmcli', {"wlanN": wlanN})
+    gw_iface = None
     if not found_inet:
-        gw.info("[nmcli] No internet via wlanN, trying wlan0 as client")
-        set_wlan0_station()
-        conn = try_connect_wlan0_known_networks()
-        if conn:
-            gw.info("[nmcli] wlan0 now has internet")
+        gw_iface = get_default_route_iface()
+        gw.debug(f"[nmcli] default route iface: {gw_iface}")
+        if gw_iface and ping(gw_iface):
             found_inet = True
-            internet_iface = "wlan0"
-            internet_ssid = conn
-        else:
-            gw.info("[nmcli] wlan0 cannot connect as client")
-            # Keep wlan0 in station mode. It will switch back to AP
-            # only when another interface provides internet.
+            internet_iface = gw_iface
+            if gw_iface in wlanN:
+                internet_ssid = wlanN[gw_iface].get("ssid")
+
+    if not internet_iface and gw_iface == "wlan0":
+        internet_ssid = gw.monitor.get_state('nmcli').get("wlan0_ssid")
 
-    # Fallback to system default route
-    gw_iface = get_default_route_iface()
-    gw.debug(f"[nmcli] default route iface: {gw_iface}")
-    if gw_iface and not internet_iface:
-        if ping(gw_iface):
-            found_inet = True
-        internet_iface = gw_iface
-        if gw_iface in wlanN:
-            internet_ssid = wlanN[gw_iface].get("ssid")
-        elif gw_iface == "wlan0":
-            internet_ssid = gw.monitor.get_state('nmcli').get("wlan0_ssid")
-            
     gw.monitor.set_states('nmcli', {
         "last_monitor_check": now_iso(),
         "internet_iface": internet_iface,
@@ -398,19 +383,24 @@ def monitor_nmcli(**kwargs):
     }
 
 def monitor_ap_only(**kwargs):
-    ap_ssid = kwargs.get("ap_ssid") or gw.resolve('[AP_SSID]')
-    ap_con = kwargs.get("ap_con") or ap_ssid or gw.resolve('[AP_CON]')
-    ap_password = kwargs.get("ap_password") or gw.resolve('[AP_PASSWORD]')
-    set_wlan0_ap(ap_con, ap_ssid, ap_password)
-    gw.monitor.set_states('nmcli', {"last_monitor_check": now_iso()})
-    state = gw.monitor.get_state('nmcli')
-    return {"wlan0_mode": state.get("wlan0_mode"), "ssid": ap_ssid}
+    """Record wlan0 status without changing configuration."""
+    gather_eth0_status()
+    status = get_wlan_status("wlan0")
+    gw.monitor.set_states('nmcli', {
+        "last_monitor_check": now_iso(),
+        "wlanN": {"wlan0": status},
+    })
+    return {"wlan0_mode": gw.monitor.get_state('nmcli').get("wlan0_mode"), "ssid": status.get("ssid")}
 
 def monitor_station_only(**kwargs):
-    set_wlan0_station()
-    gw.monitor.set_states('nmcli', {"last_monitor_check": now_iso()})
-    state = gw.monitor.get_state('nmcli')
-    return {"wlan0_mode": state.get("wlan0_mode")}
+    """Record wlan0 status without changing configuration."""
+    gather_eth0_status()
+    status = get_wlan_status("wlan0")
+    gw.monitor.set_states('nmcli', {
+        "last_monitor_check": now_iso(),
+        "wlanN": {"wlan0": status},
+    })
+    return {"wlan0_mode": gw.monitor.get_state('nmcli').get("wlan0_mode")}
 
 # --- Renderers (for dashboard, html output) ---
 
@@ -447,8 +437,12 @@ def render_nmcli():
     html = ['<div class="nmcli-report">']
     html.append("<h2>Network Manager</h2>")
     html.append(f"<b>Last monitor check:</b> {s.get('last_monitor_check') or '-'}<br>")
-    html.append(f"<b>Last config change:</b> {s.get('last_config_change') or 'Never'}<br>")
-    html.append(f"<b>Last action:</b> {s.get('last_config_action') or '-'}<br>")
+    last_action = s.get('last_config_action')
+    last_change = s.get('last_config_change')
+    if last_action and last_change:
+        html.append(f"<b>Last action:</b> {last_change} - {last_action}<br>")
+    else:
+        html.append(f"<b>Last action:</b> {last_action or '-'}<br>")
     html.append(f"<b>wlan0 mode:</b> {s.get('wlan0_mode') or '-'}<br>")
     # AP info
     wlan0_info = device_info.get('wlan0', {})
@@ -477,21 +471,29 @@ def render_nmcli():
 
     # All wlanN and relevant info (including disconnected/disabled)
     html.append(f"<b>WLANN interfaces:</b> {wlan_count}<br>")
-    html.append('<table style="border-collapse:collapse;margin-top:4px;"><tr>'
-                '<th>iface</th><th>SSID</th><th>Connected</th><th>INET</th><th>State</th><th>Driver</th><th>MAC</th></tr>')
+    html.append('<table style="border-collapse:collapse;margin-top:4px;font-size:90%;">'
+                '<tr>'
+                '<th style="border:1px solid #ccc;padding:2px 4px;">iface</th>'
+                '<th style="border:1px solid #ccc;padding:2px 4px;">SSID</th>'
+                '<th style="border:1px solid #ccc;padding:2px 4px;">Connected</th>'
+                '<th style="border:1px solid #ccc;padding:2px 4px;">INET</th>'
+                '<th style="border:1px solid #ccc;padding:2px 4px;">State</th>'
+                '<th style="border:1px solid #ccc;padding:2px 4px;">Driver</th>'
+                '<th style="border:1px solid #ccc;padding:2px 4px;">MAC</th>'
+                '</tr>')
     for dev in sorted([d for d in devices if d.startswith('wlan')]):
         st = wlanN.get(dev, {})
         dinfo = device_info.get(dev, {})
         gw_mark = ' <b>(gw)</b>' if dev == internet_iface else ''
         html.append(
             f"<tr>"
-            f"<td>{dev}</td>"
-            f"<td>{st.get('ssid') or '-'}</td>"
-            f"<td>{_color_icon(st.get('connected'))} {st.get('connected') if 'connected' in st else dinfo.get('state','-')}</td>"
-            f"<td>{_color_icon(st.get('inet')) if 'inet' in st else _color_icon(dinfo.get('state')=='connected')} {st.get('inet') if 'inet' in st else '-'}{gw_mark}</td>"
-            f"<td>{dinfo.get('state','-')}</td>"
-            f"<td>{dinfo.get('driver','-')}</td>"
-            f"<td>{dinfo.get('mac','-')}</td>"
+            f'<td style="border:1px solid #ccc;padding:2px 4px;">{dev}</td>'
+            f'<td style="border:1px solid #ccc;padding:2px 4px;">{st.get("ssid") or "-"}</td>'
+            f'<td style="border:1px solid #ccc;padding:2px 4px;">{_color_icon(st.get("connected"))} {st.get("connected") if "connected" in st else dinfo.get("state", "-")}</td>'
+            f'<td style="border:1px solid #ccc;padding:2px 4px;">{_color_icon(st.get("inet")) if "inet" in st else _color_icon(dinfo.get("state")=="connected")} {st.get("inet") if "inet" in st else "-"}{gw_mark}</td>'
+            f'<td style="border:1px solid #ccc;padding:2px 4px;">{dinfo.get("state", "-")}</td>'
+            f'<td style="border:1px solid #ccc;padding:2px 4px;">{dinfo.get("driver", "-")}</td>'
+            f'<td style="border:1px solid #ccc;padding:2px 4px;">{dinfo.get("mac", "-")}</td>'
             f"</tr>"
         )
     html.append('</table>')
@@ -506,5 +508,38 @@ def render_nmcli():
     else:
         html.append(f"<b>Internet via:</b> <span style='color:#b00;'>No gateway detected</span><br>")
 
+    # --- Command box ---
+    html.append(_render_run_form())
+
     html.append("</div>")
     return "\n".join(html)
+
+
+def _render_run_form(cmd: str = "", output: str = "") -> str:
+    url = gw.web.app.build_url("run") if hasattr(gw, "web") else "run"
+    form = [
+        f"<form method='post' action='{url}' style='margin-top:8px;'>",
+        f"<input type='text' name='cmd' value='{html.escape(cmd, quote=True)}' placeholder='nmcli arguments' style='width:70%;'>",
+        "<button type='submit'>Run</button>",
+        "</form>",
+    ]
+    if output:
+        form.append(f"<pre>{html.escape(output)}</pre>")
+    return "".join(form)
+
+
+def view_get_run(cmd: str = ""):
+    """Display nmcli command form."""
+    return _render_run_form(cmd)
+
+
+def view_post_run(cmd: str = ""):
+    """Execute nmcli command and show output."""
+    cmd = cmd or request.forms.get("cmd", "")
+    output = ""
+    if cmd:
+        try:
+            output = nmcli(*shlex.split(cmd))
+        except Exception as e:
+            output = f"Error: {e}"
+    return _render_run_form(cmd, output)
