@@ -7,15 +7,15 @@ from datetime import datetime
 from typing import Iterable, Optional, Sequence
 from gway import gw
 
-_db_conn = None
-
 def open_db():
     """Return connection to the OCPP database, initializing tables."""
-    global _db_conn
-    if _db_conn is None:
-        _db_conn = gw.sql.open_connection("work/ocpp.sqlite")
-        _init_db(_db_conn)
-    return _db_conn
+    conn = gw.sql.open_connection(
+        "work/ocpp.db", sql_engine="duckdb"
+    )
+    if not getattr(open_db, "_init", False):
+        _init_db(conn)
+        setattr(open_db, "_init", True)
+    return conn
 
 def _init_db(conn):
     gw.sql.execute(
@@ -61,6 +61,24 @@ def _init_db(conn):
         """,
         connection=conn,
     )
+    gw.sql.execute(
+        """
+        CREATE TABLE IF NOT EXISTS connections(
+            charger_id TEXT PRIMARY KEY,
+            connected INTEGER,
+            last_heartbeat TEXT,
+            status TEXT,
+            error_code TEXT,
+            info TEXT
+        )
+        """,
+        connection=conn,
+    )
+    # track when we last received any message from the charger
+    gw.sql.execute(
+        "ALTER TABLE connections ADD COLUMN IF NOT EXISTS last_msg INTEGER",
+        connection=conn,
+    )
 
 def record_transaction_start(
     charger_id: str,
@@ -84,6 +102,7 @@ def record_transaction_start(
             charger_timestamp,
         ),
     )
+    record_last_msg(charger_id, start_time)
 
 def record_transaction_stop(
     charger_id: str,
@@ -107,6 +126,7 @@ def record_transaction_stop(
             transaction_id,
         ),
     )
+    record_last_msg(charger_id, stop_time)
 
 def record_meter_value(charger_id: str, transaction_id: int, timestamp: int, measurand: str, value: float, unit: str = "", context: str = ""):
     conn = open_db()
@@ -115,6 +135,7 @@ def record_meter_value(charger_id: str, transaction_id: int, timestamp: int, mea
         connection=conn,
         args=(charger_id, transaction_id, int(timestamp), measurand, float(value), unit, context),
     )
+    record_last_msg(charger_id, timestamp)
 
 def record_error(charger_id: str, status: str, error_code: str = "", info: str = ""):
     conn = open_db()
@@ -124,6 +145,75 @@ def record_error(charger_id: str, status: str, error_code: str = "", info: str =
         connection=conn,
         args=(charger_id, status, error_code, info, ts),
     )
+    record_last_msg(charger_id, ts)
+
+def set_connection_status(charger_id: str, connected: bool):
+    """Mark charger connection as active or inactive."""
+    conn = open_db()
+    gw.sql.execute(
+        "DELETE FROM connections WHERE charger_id=?",
+        connection=conn,
+        args=(charger_id,),
+    )
+    gw.sql.execute(
+        "INSERT INTO connections(charger_id, connected, last_msg) VALUES (?, ?, ?)",
+        connection=conn,
+        args=(charger_id, 1 if connected else 0, int(time.time())),
+    )
+
+def record_heartbeat(charger_id: str, timestamp: str):
+    conn = open_db()
+    gw.sql.execute(
+        "UPDATE connections SET last_heartbeat=? WHERE charger_id=?",
+        connection=conn,
+        args=(timestamp, charger_id),
+    )
+    record_last_msg(charger_id)
+
+def update_status(charger_id: str, status: str = None, error_code: str = None, info: str = None):
+    conn = open_db()
+    gw.sql.execute(
+        "UPDATE connections SET status=?, error_code=?, info=? WHERE charger_id=?",
+        connection=conn,
+        args=(status, error_code, info, charger_id),
+    )
+
+def clear_status(charger_id: str):
+    conn = open_db()
+    gw.sql.execute(
+        "UPDATE connections SET status=NULL, error_code=NULL, info=NULL WHERE charger_id=?",
+        connection=conn,
+        args=(charger_id,),
+    )
+
+def record_last_msg(charger_id: str, timestamp: int | None = None):
+    """Update the last_msg timestamp for a charger."""
+    conn = open_db()
+    ts = int(timestamp or time.time())
+    gw.sql.execute(
+        "UPDATE connections SET last_msg=? WHERE charger_id=?",
+        connection=conn,
+        args=(ts, charger_id),
+    )
+
+def get_connection(charger_id: str):
+    conn = open_db()
+    rows = gw.sql.execute(
+        "SELECT connected, last_heartbeat, status, error_code, info, last_msg FROM connections WHERE charger_id=?",
+        connection=conn,
+        args=(charger_id,),
+    )
+    if rows:
+        c, hb, st, ec, info, lm = rows[0]
+        return {
+            "connected": bool(c),
+            "last_heartbeat": hb,
+            "status": st,
+            "error_code": ec,
+            "info": info,
+            "last_msg": lm,
+        }
+    return None
 
 def get_summary():
     """Return summary rows per charger."""
@@ -211,6 +301,104 @@ def iter_transactions(
     args.extend([int(limit), int(offset)])
     return gw.sql.execute(sql, connection=conn, args=tuple(args))
 
+def get_active_transaction(charger_id: str):
+    conn = open_db()
+    rows = gw.sql.execute(
+        "SELECT charger_id, transaction_id, start_time, meter_start, id_tag FROM transactions WHERE charger_id=? AND stop_time IS NULL ORDER BY start_time DESC LIMIT 1",
+        connection=conn,
+        args=(charger_id,),
+    )
+    if rows:
+        c, tid, st, ms, tag = rows[0]
+        return {
+            "charger_id": c,
+            "transactionId": tid,
+            "startTime": st,
+            "meterStart": ms,
+            "idTag": tag,
+        }
+    return None
+
+def get_active_transactions():
+    conn = open_db()
+    rows = gw.sql.execute(
+        "SELECT charger_id, transaction_id, start_time, meter_start, id_tag FROM transactions WHERE stop_time IS NULL",
+        connection=conn,
+    )
+    result = {}
+    for c, tid, st, ms, tag in rows:
+        result[c] = {
+            "charger_id": c,
+            "transactionId": tid,
+            "startTime": st,
+            "meterStart": ms,
+            "idTag": tag,
+        }
+    return result
+
+def get_active_chargers() -> list[str]:
+    """Return list of charger IDs currently marked as connected."""
+    conn = open_db()
+    rows = gw.sql.execute(
+        "SELECT charger_id FROM connections WHERE connected=1",
+        connection=conn,
+    )
+    return [r[0] for r in rows]
+
+def get_meter_values(charger_id: str, transaction_id: int):
+    conn = open_db()
+    rows = gw.sql.execute(
+        "SELECT timestamp, measurand, value, unit, context FROM meter_values WHERE charger_id=? AND transaction_id=? ORDER BY timestamp",
+        connection=conn,
+        args=(charger_id, transaction_id),
+    )
+    grouped = {}
+    for ts, meas, val, unit, ctx in rows:
+        entry = grouped.setdefault(
+            ts,
+            {
+                "timestamp": ts,
+                "timestampStr": datetime.utcfromtimestamp(int(ts)).isoformat() + "Z",
+                "sampledValue": [],
+            },
+        )
+        entry["sampledValue"].append(
+            {
+                "measurand": meas,
+                "value": val,
+                "unit": unit,
+                "context": ctx,
+            }
+        )
+    return list(grouped.values())
+
+def get_latest_meter_value(charger_id: str, transaction_id: int):
+    """Return the most recent Energy.Active.Import.Register value in kWh."""
+    conn = open_db()
+    rows = gw.sql.execute(
+        """
+        SELECT value, unit FROM meter_values
+        WHERE charger_id=? AND transaction_id=?
+          AND measurand='Energy.Active.Import.Register'
+        ORDER BY timestamp DESC LIMIT 1
+        """,
+        connection=conn,
+        args=(charger_id, transaction_id),
+    )
+    if rows:
+        val, unit = rows[0]
+        try:
+            fval = float(val)
+            if unit == 'Wh':
+                fval /= 1000.0
+            return fval
+        except Exception:
+            try:
+                return float(val)
+            except Exception:
+                return None
+    return None
+
 def get_meter_series(chargers: Sequence[str], *, start: int = None, end: int = None):
     """Return dict of charger_id -> list of (timestamp, kWh)."""
     conn = open_db()
@@ -236,7 +424,7 @@ def list_chargers() -> list[str]:
     """Return list of distinct charger_ids."""
     conn = open_db()
     rows = gw.sql.execute(
-        "SELECT DISTINCT charger_id FROM transactions ORDER BY charger_id",
+        "SELECT charger_id FROM connections UNION SELECT DISTINCT charger_id FROM transactions ORDER BY charger_id",
         connection=conn,
     )
     return [r[0] for r in rows]
