@@ -6,6 +6,8 @@ import threading
 import time
 import html
 import re
+import ast
+import importlib.util
 from pathlib import Path
 from io import StringIO
 import unittest
@@ -31,7 +33,7 @@ def build(
     bump: bool = False,
     dist: bool = False,
     twine: bool = False,
-    help_db: bool = True,
+    help_db: bool = False,
     projects: bool = False,
     git: bool = False,
     all: bool = False,
@@ -61,7 +63,6 @@ def build(
         bump = True
         dist = True
         twine = True
-        help_db = True
         git = True
         projects = True
 
@@ -75,8 +76,7 @@ def build(
         if status.stdout.strip():
             gw.abort("Git repository is not clean. Commit or stash changes before building.")
 
-    if help_db:
-        build_help_db()
+
 
     if projects:
         project_dir = gw.resource("projects")
@@ -263,8 +263,6 @@ def build(
 
     if git:
         files_to_add = ["VERSION", "BUILD", "pyproject.toml", "CHANGELOG.rst"]
-        if help_db:
-            files_to_add.append("data/help.sqlite")
         if projects:
             files_to_add.append("README.rst")
         subprocess.run(["git", "add"] + files_to_add, check=True)
@@ -275,81 +273,10 @@ def build(
 
 
 def build_help_db():
-    with gw.sql.open_connection(datafile="data/help.sqlite") as cursor:
-        cursor.execute("DROP TABLE IF EXISTS help")
-        cursor.execute("""
-            CREATE VIRTUAL TABLE help USING fts5(
-                project, function, signature, docstring, source, todos, tokenize='porter')
-        """)
-
-        for dotted_path in _walk_projects("projects"):
-            try:
-                project_obj = gw.load_project(dotted_path)
-                for fname in dir(project_obj):
-                    if fname.startswith("_"):
-                        continue
-                    func = getattr(project_obj, fname, None)
-                    if not callable(func):
-                        continue
-                    raw_func = getattr(func, "__wrapped__", func)
-                    doc = inspect.getdoc(raw_func) or ""
-                    sig = str(inspect.signature(raw_func))
-                    try:
-                        source = "".join(inspect.getsourcelines(raw_func)[0])
-                    except OSError:
-                        source = ""
-                    todos = _extract_todos(source)
-                    cursor.execute("INSERT INTO help VALUES (?, ?, ?, ?, ?, ?)",
-                                   (dotted_path, fname, sig, doc, source, "\n".join(todos)))
-            except Exception as e:
-                gw.warning(f"Skipping project {dotted_path}: {e}")
-
-        # Add builtin functions under synthetic project "builtin"
-        for name, func in gw._builtins.items():
-            raw_func = getattr(func, "__wrapped__", func)
-            doc = inspect.getdoc(raw_func) or ""
-            sig = str(inspect.signature(raw_func))
-            try:
-                source = "".join(inspect.getsourcelines(raw_func)[0])
-            except OSError:
-                source = ""
-            todos = _extract_todos(source)
-
-            cursor.execute("INSERT INTO help VALUES (?, ?, ?, ?, ?, ?)",
-                           ("builtin", name, sig, doc, source, "\n".join(todos)))
-
-        cursor.execute("COMMIT")
+    """Compatibility wrapper that delegates to :mod:`help_db`."""
+    return gw.help_db.build(update=True)
 
 
-def _walk_projects(base="projects"):
-    """Yield all project modules as dotted paths."""
-    for dirpath, _, filenames in os.walk(base):
-        for fname in filenames:
-            if not fname.endswith(".py") or fname.startswith("_"):
-                continue
-            rel_path = os.path.relpath(os.path.join(dirpath, fname), base)
-            dotted = rel_path.replace(os.sep, ".").removesuffix(".py")
-            yield dotted
-
-
-def _extract_todos(source):
-    todos = []
-    lines = source.splitlines()
-    current = []
-    for line in lines:
-        stripped = line.strip()
-        if "# TODO" in stripped:
-            if current:
-                todos.append("\n".join(current))
-            current = [stripped]
-        elif current and (stripped.startswith("#") or not stripped):
-            current.append(stripped)
-        elif current:
-            todos.append("\n".join(current))
-            current = []
-    if current:
-        todos.append("\n".join(current))
-    return todos
 
 
 def loc(*paths):
@@ -457,6 +384,74 @@ def changes(*, files=None, staged=False, context=3, max_bytes=200_000, clip=Fals
     return gw.hub.changes(files=files, staged=staged, context=context, max_bytes=max_bytes, clip=clip)
 
 
+def build_requirements(func):
+    """Generate a requirements file for ``func`` and its callees."""
+
+    if isinstance(func, str):
+        module_name, attr = func.rsplit(".", 1)
+        mod = __import__(module_name, fromlist=[attr])
+        func = getattr(mod, attr)
+
+    visited = set()
+    modules = set()
+
+    def is_stdlib(name: str) -> bool:
+        try:
+            spec = importlib.util.find_spec(name)
+        except ModuleNotFoundError:
+            return False
+        if not spec or not spec.origin:
+            return True
+        path = spec.origin or ""
+        return "site-packages" not in path and "dist-packages" not in path
+
+    def gather(f):
+        if not callable(f) or f in visited:
+            return
+        visited.add(f)
+        try:
+            source = inspect.getsource(f)
+        except Exception:
+            return
+        tree = ast.parse(source)
+        globals_ = getattr(f, "__globals__", {})
+        for node in ast.walk(tree):
+            if isinstance(node, ast.Import):
+                for alias in node.names:
+                    name = alias.name.split(".")[0]
+                    if not is_stdlib(name):
+                        modules.add(name)
+            elif isinstance(node, ast.ImportFrom):
+                if node.module:
+                    name = node.module.split(".")[0]
+                    if not is_stdlib(name):
+                        modules.add(name)
+            elif isinstance(node, ast.Call):
+                target = None
+                if isinstance(node.func, ast.Name):
+                    target = globals_.get(node.func.id)
+                elif isinstance(node.func, ast.Attribute) and isinstance(node.func.value, ast.Name):
+                    base = globals_.get(node.func.value.id)
+                    if base is not None:
+                        target = getattr(base, node.func.attr, None)
+                if inspect.isfunction(target):
+                    gather(target)
+                elif inspect.ismodule(target):
+                    name = target.__name__.split(".")[0]
+                    if not is_stdlib(name):
+                        modules.add(name)
+
+    gather(func)
+
+    qualname = getattr(func, "__qualname__", getattr(func, "__name__", "func"))
+    dest = Path("work") / "release" / qualname.replace(".", "_")
+    dest.mkdir(parents=True, exist_ok=True)
+    req_file = dest / "requirements.txt"
+    req_file.write_text("\n".join(sorted(modules)) + "\n", encoding="utf-8")
+    gw.info(f"Wrote requirements to {req_file}")
+    return req_file
+
+
 def _last_changelog_build():
     path = Path("CHANGELOG.rst")
     if not path.exists():
@@ -772,7 +767,7 @@ def view_test_cache():
     cov_table += "</table>"
 
     log_block = (
-        "<div id='test-log' data-gw-render='test_log' data-gw-refresh='2'>"
+        "<div id='test-log' gw-render='test_log' gw-refresh='2'>"
         + render_test_log()
         + "</div>"
     )
