@@ -1,4 +1,21 @@
 # file: projects/web/app.py
+"""Web application dispatcher for GWAY.
+
+`setup_app` registers a project and exposes any ``view_*`` functions under
+``/project`` for HTML responses, ``api_*`` under ``/api/project`` for JSON, and
+``render_*`` under ``/render/project/<view>/<hash>`` for fragment updates.
+
+Functions can be specialized by HTTP method (``view_get_*``/``view_post_*``) and
+multiple view names may be combined with ``+`` in the path to build a mashup.
+``render_*`` functions may return HTML or JSON and are ideal for dynamic
+refreshes via ``render.js``.
+CSS and JavaScript from enabled projects are bundled via
+``web.static.collect``. ``setup_app`` defaults to ``mode='collect'`` so pages
+load ``/shared/global.css`` and ``/shared/global.js`` automatically. Add
+``<view>.css`` or ``<view>.js`` (without the ``view_`` prefix) to
+``data/static/<project>`` for view-specific assets and avoid manual ``<link>`` or
+``<script>`` tags unless ``mode='manual'`` is requested.
+"""
 
 import os
 from urllib.parse import urlencode
@@ -18,6 +35,9 @@ _enabled = set()
 _registered_routes: set[tuple[str, str]] = set()
 _fresh_mtime = None
 _fresh_dt = None
+_static_route = "static"
+_shared_route = "shared"
+_default_include_mode = "collect"
 UPLOAD_MB = 100
 
 def _refresh_fresh_date():
@@ -80,6 +100,7 @@ def setup_app(project,
     shared="shared",
     css="global",           # Default CSS (without .css extension)
     js="global",            # Default JS  (without .js extension)
+    mode="collect",        # collect | manual | embedded
     auth="disabled",       # Accept "optional"/"disabled" words to disable
     engine="bottle",
     **setup_kwargs,
@@ -87,10 +108,11 @@ def setup_app(project,
     """
     Setup Bottle web application with symmetrical static/shared public folders.
     ``project`` may be a single name or sequence of fallback names. The first
-    project found is loaded and used. CSS/JS params are used as the only static
-    includes.
+    project found is loaded and used. ``mode`` controls how CSS/JS files are
+    included: ``collect`` (default) uses bundled files, ``manual`` links each
+    file individually, and ``embedded`` inlines the contents into the page.
     """
-    global _ver, _homes, _enabled
+    global _ver, _homes, _enabled, _static_route, _shared_route
 
     auth_required = str(auth).strip().lower() not in {
         "none", "false", "disabled", "optional"
@@ -101,6 +123,11 @@ def setup_app(project,
 
     _ver = _ver or gw.version()
     bottle.BaseRequest.MEMFILE_MAX = UPLOAD_MB * 1024 * 1024
+    if static:
+        _static_route = static
+    if shared:
+        _shared_route = shared
+    include_mode = str(mode or _default_include_mode).strip().lower()
 
     project_names = gw.cast.to_list(project)
     if not project_names:
@@ -227,6 +254,7 @@ def setup_app(project,
 
         def view_dispatch(view):
             nonlocal home, views
+            request.environ['gw.include_mode'] = include_mode
             if (
                 unauth := _maybe_auth(
                     "Unauthorized: You are not permitted to view this page."
@@ -263,6 +291,7 @@ def setup_app(project,
                     return gw.web.error.redirect(
                         f"View not found: {method_func_name} or {generic_func_name} in {project}"
                     )
+                _record_includes(view_func)
 
                 try:
                     content = view_func(*args, **kwargs)
@@ -293,11 +322,18 @@ def setup_app(project,
 
             final_content = "".join(contents)
             media_origin = "/shared" if shared else ("static" if static else "")
+            if include_mode == "collect":
+                css_files = (f"{media_origin}/{css}.css",) if css else None
+                js_files = (f"{media_origin}/{js}.js",) if js else None
+            else:
+                css_files = None
+                js_files = None
             return render_template(
                 title="GWAY - " + " + ".join(titles),
                 content=final_content,
-                css_files=(f"{media_origin}/{css}.css",),
-                js_files=(f"{media_origin}/{js}.js",),
+                css_files=css_files,
+                js_files=js_files,
+                mode=include_mode,
             )
 
         def index_dispatch():
@@ -430,6 +466,7 @@ def setup_app(project,
                 if not callable(view_func):
                     return gw.web.error.redirect(
                         f"View not found: {method_func_name} or {generic_func_name} in {project}")
+                _record_includes(view_func)
 
                 try:
                     content = view_func(*args, **kwargs)
@@ -496,7 +533,7 @@ def build_url(*args, **kwargs):
         url += "?" + urlencode(kwargs)
     return url
 
-def render_template(*, title="GWAY", content="", css_files=None, js_files=None):
+def render_template(*, title="GWAY", content="", css_files=None, js_files=None, mode=None):
     global _ver
     version = _ver = _ver or gw.version()
     fresh = _format_fresh(_refresh_fresh_date())
@@ -507,27 +544,58 @@ def render_template(*, title="GWAY", content="", css_files=None, js_files=None):
         except Exception:
             build = ""
 
-    css_files = gw.cast.to_list(css_files)
+    mode = str(mode or getattr(request, 'environ', {}).get('gw.include_mode') or _default_include_mode).lower()
+    extra_css = request.environ.get("gw.include_css", [])
+    extra_js = request.environ.get("gw.include_js", [])
+
+    css_files = [c for c in gw.cast.to_list(css_files) if c] if mode == "collect" else []
+    js_files = [j for j in gw.cast.to_list(js_files) if j] if mode == "collect" else []
     theme_css = None
     if is_setup('web.nav'):
         try:
             theme_css = gw.web.nav.active_style()
         except Exception:
             theme_css = None
-    # <<< Patch: APPEND, don't prepend! >>>
-    if theme_css and theme_css not in css_files:
-        css_files.append(theme_css)
 
     css_links = ""
-    if css_files:
+    js_links = ""
+
+    if mode == "collect":
+        if theme_css and theme_css not in css_files:
+            css_files.append(theme_css)
         for href in css_files:
             css_links += f'<link rel="stylesheet" href="{href}">\n'
-
-    js_files = gw.cast.to_list(js_files)
-    js_links = ""
-    if js_files:
         for src in js_files:
             js_links += f'<script src="{src}"></script>\n'
+    elif mode == "manual":
+        css_refs = [f"/{_static_route}/" + str(p).lstrip("/") for p in extra_css]
+        if theme_css:
+            css_refs.append(theme_css)
+        for href in css_refs:
+            css_links += f'<link rel="stylesheet" href="{href}">\n'
+        for src in [f"/{_static_route}/" + str(p).lstrip("/") for p in extra_js]:
+            js_links += f'<script src="{src}"></script>\n'
+    else:  # embedded
+        css_paths = [gw.resource("data", "static", p) for p in extra_css]
+        if theme_css:
+            parts = theme_css.lstrip("/").split("/")
+            if parts and parts[0] == _static_route:
+                css_paths.append(gw.resource("data", "static", *parts[1:]))
+            elif parts and parts[0] == _shared_route:
+                css_paths.append(gw.resource("work", "shared", *parts[1:]))
+        for path in css_paths:
+            try:
+                with open(path, "r", encoding="utf-8") as f:
+                    css_links += f"<style>\n{f.read()}\n</style>\n"
+            except Exception:
+                gw.debug(f"Missing CSS to embed: {path}")
+        js_paths = [gw.resource("data", "static", p) for p in extra_js]
+        for path in js_paths:
+            try:
+                with open(path, "r", encoding="utf-8") as f:
+                    js_links += f"<script>\n{f.read()}\n</script>\n"
+            except Exception:
+                gw.debug(f"Missing JS to embed: {path}")
 
     favicon = f'<link rel="icon" href="/favicon.ico" type="image/x-icon" />'
     credits = f'''
@@ -536,6 +604,7 @@ def render_template(*, title="GWAY", content="", css_files=None, js_files=None):
         <a href="https://pypi.org">PyPI</a> and <a href="https://github.com/arthexis/gway">Github</a>.</p>
     '''
     nav = gw.web.nav.render(homes=_homes, links=_links) if is_setup('web.nav') else ""
+    nav_side = gw.web.nav.side() if is_setup('web.nav') else "left"
 
     debug_html = ""
     if getattr(gw, "debug_enabled", False):
@@ -573,8 +642,14 @@ def render_template(*, title="GWAY", content="", css_files=None, js_files=None):
         </head>
         <body>
             <div class="page-wrap">
-                <div class="layout">
+                <div class="layout{{' nav-right' if nav_side == 'right' else (' nav-top' if nav_side == 'top' else '')}}">
+                    % if nav_side == 'right':
+                    <main>{{!message_html}}{{!content}}</main>{{!nav}}
+                    % elif nav_side == 'top':
                     {{!nav}}<main>{{!message_html}}{{!content}}</main>
+                    % else:
+                    {{!nav}}<main>{{!message_html}}{{!content}}</main>
+                    % end
                 </div>
                 <footer><p>This website was <strong>built</strong>, <strong>tested</strong>
                     and <strong>released</strong> with <a href="https://arthexis.com">GWAY</a>
@@ -618,6 +693,15 @@ def add_route(app, rule: str, method, callback):
         _registered_routes.add(key)
         app.route(rule, method=m)(callback)
 
+def _record_includes(func):
+    """Record CSS/JS includes for the current request if present."""
+    css = getattr(func, "_include_css", [])
+    js = getattr(func, "_include_js", [])
+    if css:
+        request.environ.setdefault("gw.include_css", set()).update(css)
+    if js:
+        request.environ.setdefault("gw.include_js", set()).update(js)
+
 def is_setup(project_name):
     global _enabled
     return project_name in _enabled
