@@ -7,6 +7,7 @@ from gway import gw
 from bottle import request
 
 _forced_style = None
+_side = "left"
 
 
 def render(*, homes=None, links=None):
@@ -195,10 +196,12 @@ def render(*, homes=None, links=None):
         data_attr = ""
         if render_compass_exists:
             data_attr = (
-                ' data-gw-render="compass" data-gw-double-click="refresh"'
+                ' gw-render="compass" gw-double-click="refresh"'
             )
         compass = f'<div class="compass"{data_attr}>{compass}</div>'
 
+    if _side == "top":
+        return f"<nav class='top-bar'><ul class='top-links'>{links_html}</ul>{search_box}</nav>"
     return f"<aside>{search_box}<ul>{links_html}</ul><br>{compass}{toggle}</aside>"
 
 
@@ -521,13 +524,25 @@ def list_styles(project=None):
     return styles
 
 
-def setup_app(*, app=None, style=None, **_):
-    """Optional hook to set a default style when the project is added.
+def setup_app(*, app=None, style=None, side="left", **_):
+    """Optional hook to set a default style and nav side when the project is added.
 
     Pass ``style='random'`` to select a random theme on each request.
+    Use ``side='right'`` to place the navigation on the right side or ``side='top'``
+    for a horizontal bar with drop-down menus.
     """
-    global _forced_style
+    global _forced_style, _side
     if style:
         _forced_style = style
         gw.info(f"web.nav forced style: {style}")
+    if side in {"left", "right", "top"}:
+        _side = side
+        gw.info(f"web.nav side set to {side}")
+    else:
+        gw.error(f"Invalid nav side: {side}")
     return app
+
+
+def side() -> str:
+    """Return the configured navigation side ('left', 'right' or 'top')."""
+    return _side
