@@ -4,12 +4,19 @@ from pathlib import Path
 from unittest.mock import patch
 from bs4 import BeautifulSoup
 
-app_path = Path(__file__).resolve().parents[1] / "projects" / "web" / "app.py"
-spec = importlib.util.spec_from_file_location("webapp", app_path)
-webapp = importlib.util.module_from_spec(spec)
-spec.loader.exec_module(webapp)
-
 class RenderTemplateThemeCssTests(unittest.TestCase):
+    @staticmethod
+    def _load_webapp():
+        app_path = Path(__file__).resolve().parents[1] / "projects" / "web" / "app.py"
+        spec = importlib.util.spec_from_file_location("webapp", app_path)
+        module = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(module)
+        return module
+
+    @classmethod
+    def setUpClass(cls):
+        global webapp
+        webapp = cls._load_webapp()
     def test_base_and_theme_css_links(self):
         dummy_theme = "/static/theme.css"
         with patch.object(webapp.gw.web.nav, 'active_style', return_value=dummy_theme), \
